@@ -1,7 +1,7 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 use Exporter 'import';
 our @EXPORT_OK = qw( generate portrait landscape north east south west );
@@ -42,7 +42,10 @@ sub generate {
 
     } elsif ($args{theta} == -90) {
 
-        $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
+        $args{data} = [ CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
+        $args{data} = ($args{pinhead} and !$args{headless})
+            ? [ map [ @$_[1 .. $#$_], $_->[0] ], @{ $args{data} } ]
+            : [ map [ CORE::reverse @$_ ], @{ $args{data} } ];
 
     } elsif ($args{theta} == 90) { # east
 
@@ -59,7 +62,9 @@ sub generate {
 
     } elsif ($args{theta} == 180) {
 
-        $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ $args{data} } ];
+        $args{data} = ($args{pinhead} and !$args{headless})
+            ? [ map [ CORE::reverse @$_ ], @{ $args{data} }[1 .. $#{ $args{data} }], $args{data}[0] ]
+            : [ map [ CORE::reverse @$_ ], CORE::reverse @{ $args{data} } ];
 
     } elsif ($args{theta} == -270) { # west
 
@@ -110,33 +115,17 @@ sub _process {
 
         for my $col (0 .. $#{ $data->[$row] }) {
             my $tag = (!$row and !($args->{headless} or $args->{matrix})) ? 'th' : 'td';
-            my $val = $data->[$row][$col];
-
-            # -rowX
-            my $attr = $args->{$tag};
-            if (exists $args->{"-row$row"}) {
-                $args->{"-row$row"} = [ $args->{"-row$row"} ] unless ref( $args->{"-row$row"} ) eq 'ARRAY';
-                for (@{ $args->{"-row$row"} }) {
-                    if (ref($_) eq 'CODE') {
-                        $val = $_->($val);
-                    } elsif (ref($_) eq 'HASH') {
-                        $attr = $_;
-                    }
-                }
-            }
+            my ( $val, $attr ) = _expand_code_or_hash( $args->{$tag}, $data->[$row][$col] );
+            $args->{$tag} = [ $args->{$tag} ] unless ref( $args->{$tag} ) eq 'ARRAY';
 
             # -colX
-            unless (!$args->{matrix} and $row == 0) {
-                if (exists $args->{"-col$col"}) {
-                    $args->{"-col$col"} = [ $args->{"-col$col"} ] unless ref( $args->{"-col$col"} ) eq 'ARRAY';
-                    for (@{ $args->{"-col$col"} }) {
-                        if (ref($_) eq 'CODE') {
-                            $val = $_->($val);
-                        } elsif (ref($_) eq 'HASH') {
-                            $attr = $_;
-                        }
-                    }
-                }
+            if (exists $args->{"-col$col"}) {
+                ( $val, $attr ) = _expand_code_or_hash( $args->{"-col$col"}, $val );
+            }
+
+            # -rowX (overides -colX)
+            if (exists $args->{"-row$row"}) {
+                ( $val, $attr ) = _expand_code_or_hash( $args->{"-row$row"}, $val );
             }
 
             # --empty
@@ -162,18 +151,42 @@ sub _process {
 
 sub _make_table {
     my %args = @_;
-    $args{$_} ||= {} for qw( table tr thead tbody tfoot );
 
-    if ($args{tgroups}) {
-        if (scalar @{ $args{data} } > 2) {
-            # replace last row between 1st and 2nd rows
-            splice @{ $args{data} }, 1, 0, pop @{ $args{data} };
-        } else {
-            delete $args{tgroups};
-        }
+    for (qw( table tr thead tbody tfoot )) {
+        delete $args{$_} unless ref($args{$_}) eq 'HASH';
     }
 
+    my @cdata = ( _caption( %args ), _colgroup( %args ) );
+
+    if ($args{tgroups}) {
+
+        my ($head, @body) = @{ $args{data} };
+        my $foot = pop @body if $args{tgroups} > 1 and scalar @{ $args{data} } > 2;
+
+        my $head_row  =     { tag => 'tr', attr => $args{tr}, cdata => $head };
+        my $foot_row  =     { tag => 'tr', attr => $args{tr}, cdata => $foot };
+        my @body_rows = map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @body;
+
+        push @cdata, (
+            { tag => 'thead', attr => $args{thead}, cdata => $head_row },
+            ( $foot ? { tag => 'tfoot', attr => $args{tfoot}, cdata => $foot_row } : () ),
+            { tag => 'tbody', attr => $args{tbody}, cdata => [@body_rows] }
+        );
+
+    } else {
+        push @cdata, map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @{ $args{data} };
+    }
+
+    my $encodes = exists $args{encodes} ? $args{encodes} : '';
+    my $auto = HTML::AutoTag->new( encodes => $encodes, indent => $args{indent}, level => $args{level} );
+    return $auto->tag( tag => 'table', attr => $args{table}, cdata => [ @cdata ] );
+}
+
+sub _caption {
+    my %args = @_;
+
     my $caption;
+
     if (ref($args{caption}) eq 'HASH') {
         (my $cdata) = keys %{ $args{caption} };
         (my $attr)  = values %{ $args{caption} };
@@ -181,9 +194,16 @@ sub _make_table {
     } elsif (defined $args{caption} ) {
         $caption = { tag => 'caption', cdata => $args{caption} };
     } 
+    
+    return $caption ? $caption : ();
+}
+
+sub _colgroup {
+    my %args = @_;
 
     my @colgroup;
     $args{col} = [ $args{col} ] if ref($args{col}) eq 'HASH';
+
     if (ref($args{col}) eq 'ARRAY') {
 
         @colgroup = ({
@@ -200,25 +220,7 @@ sub _make_table {
         }
     }
 
-    my ($head, $foot, @body) = @{ $args{data} };
-    my $head_row  = { tag => 'tr', attr => $args{tr}, cdata => $head };
-    my $foot_row  = { tag => 'tr', attr => $args{tr}, cdata => $foot };
-    my @body_rows = map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @body;
-
-    my $encodes = exists $args{encodes} ? $args{encodes} : '';
-    my $auto = HTML::AutoTag->new( encodes => $encodes, indent => $args{indent} );
-
-    return $auto->tag(
-        tag => 'table',
-        attr => $args{table},
-        cdata => [
-            ( ref( $caption ) ? $caption : () ),
-            ( @colgroup ? @colgroup : () ),
-            ( $args{tgroups} ? { tag => 'thead', attr => $args{thead}, cdata => $head_row }  : $head_row ),
-            ( $args{tgroups} ? { tag => 'tfoot', attr => $args{tfoot}, cdata => $foot_row }  : $foot ? $foot_row : () ),
-              $args{tgroups} ? { tag => 'tbody', attr => $args{tbody}, cdata => [@body_rows] } : @body_rows
-        ],
-    );
+    return @colgroup;
 }
 
 sub _args {
@@ -260,6 +262,20 @@ sub _args {
     return ( $self, Clone::clone($data), $args );
 }
 
+sub _expand_code_or_hash {
+    my ( $thingy, $val ) = @_;
+    my $attr;
+    $thingy = [ $thingy ] unless ref( $thingy ) eq 'ARRAY';
+    for (@{ $thingy }) {
+        if (ref($_) eq 'CODE') {
+            $val = $_->($val);
+        } elsif (ref($_) eq 'HASH') {
+            $attr = $_;
+        }
+    }
+    return ( $val, $attr );
+}
+
 sub transpose   { no warnings; warn "transpose is deprecated, use landscape";       generate( @_, theta => -270, tgroups => 0 ) }
 sub earthquake  { no warnings; warn "earthquake is deprecated, use east";           generate( @_, theta =>  90, tgroups => 0 ) }
 sub reverse     { no warnings; warn "reverse is deprecated, use south with flip";   generate( @_, theta => 180, tgroups => 0 ) }
@@ -432,6 +448,16 @@ when set to any value that is defined.
 
   indent => '    '
 
+  indent => "\t"
+
+=item * C<level>
+
+Start indentation at this level. Useful for matching
+nesting styles of original HTML text that you may want
+to insert into to.
+
+  level => 4
+
 =item * C<encodes>
 
 HTML Encode contents of td tags. Defaults to empty string
@@ -449,41 +475,35 @@ Set value to undef to avoid any substitutions.
 
   empty => '&#160;'
 
-=item * C<tgroups: 0 or 1>
+=item * C<tgroups: 0, 1 or 2>
 
 Group table rows into <thead> <tfoot> and <tbody>
 sections. The <tfoot> section is always found before
 the <tbody> section. Only available for C<generate()>,
 C<portrait()> and C<mirror()>.
 
-  tgroups => 1
+When C<tgroups> is set to 2 (or higher), <tfoot> sections
+are omitted. The last row of the data is found at the end
+of the <tbody> section instead.
 
 =item * C<cache: 0 or 1>
 
-  cache => 1
-
 Preserve data after it has been processed (and loaded).
 
 =item * C<matrix: 0 or 1>
 
 Render the table with only td tags, no th tags, if true.
 
-  matrix => 1
-
 =item * C<headless: 0 or 1>
 
 Render the table with without the headings row, if true. 
 
-  headless => 1
-
 =item * C<pinhead: 0 or 1>
 
 Works in conjunction with C<theta> to produces tables with
 headings placed on sides other than the top and perserve
 data alignment for reporting readability.
 
-  pinhead => 1
-
 =item * C<headings>
 
 Apply anonymous subroutine to each cell in headings row.
@@ -556,41 +576,33 @@ Add colgroup tag(s) to the table. Use an AoH for multiple.
 =item * C<col>
 
 Add col tag(s) to the table. Use an AoH for multiple. Wraps
-tags within a colgroup tag.
-
-  col => { span => 2, style => { 'background-color' => 'orange' } }
-
-  col => [ { span => 20 }, { span => 1, class => 'end' } ]
+tags within a colgroup tag. Same usage as C<colgroup>.
 
 =item * C<table>
 
+=item * C<thead>
+
+=item * C<tfoot>
+
+=item * C<tbody>
+
+=item * C<tr>
+
 Apply these attributes to the table tag.
 
   table => { class => 'spreadsheet' }
 
-=item * C<thead>
-
-  thead => { style => 'background: color' }
-
-=item * C<tfoot>
-
-  tfoot => { style => 'background: color' }
-
-=item * C<tbody>
-
-  tbody => { style => 'background: color' }
-
-=item * C<tr>
-
   tr => { style => { background => [qw( color1 color2 )]' } }
 
 =item * C<th>
 
-  th => { style => 'background: color' }
-
 =item * C<td>
 
-  td => { style => 'background: color' }
+<th> and <td> can also accept sub refs
+
+  th => sub { uc shift }
+
+  td => [ sub { uc shift }, { class => 'foo' } ]
 
 =back
 
